/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Bonus Example - stepper.c - Text widget and arrow buttons
 *  
 *  HDR2:
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/****** Include required header files ******/

#include <X11/Intrinsic.h>  /* The Intrinsics                 */
#include <Xm/Xm.h>     /* Resource symbols               */
#include <Xm/Form.h>    /* Motif XmForm widget            */
#include <Xm/ArrowB.h>   /* Motif XmArrowButton widget     */
#include <Xm/TextF.h>   /* Motif XmTextField widget       */

/****** Declare global variables ***********/

int number;
Widget textf; 

/****** Forward declare callback routines ***/

void StepUp(); 
void StepDown(); 
void VerifyInput(); 
void NumberInput(); 

/****** Start of main() *********************/

main(argc,argv)
int  argc;
char **argv;
{
 XtAppContext  context; 
 Widget     toplevel, form, uparr, dnarr;  
 toplevel = XtAppInitialize(&context, "Stepper", NULL, 0,  
 &argc, argv, NULL, NULL, 0);
 form = XtCreateManagedWidget( "form", xmFormWidgetClass, 
 toplevel, NULL, 0);
 textf = XtCreateManagedWidget( "textf", xmTextFieldWidgetClass, 
 form, NULL, 0);
 uparr = XtCreateManagedWidget( "uparr", xmArrowButtonWidgetClass,
 form, NULL, 0);
 dnarr = XtCreateManagedWidget( "dnarr", xmArrowButtonWidgetClass,
 form, NULL, 0); 
 number = 0;        /* initialize number */
 XtAddCallback( uparr, XmNactivateCallback, StepUp, NULL);
 XtAddCallback( dnarr, XmNactivateCallback, StepDown, NULL);
 XtAddCallback( textf, XmNmodifyVerifyCallback, VerifyInput, NULL);
 XtAddCallback( textf, XmNvalueChangedCallback, NumberInput, NULL);
 if (XmVersion == 1001) { 
   XtVaSetValues(uparr, XmNleftWidget, textf, NULL);
   XtVaSetValues(dnarr, XmNleftWidget, textf, NULL);
   XtVaSetValues(dnarr, XmNtopWidget, uparr, NULL);  
 }
 XtRealizeWidget(toplevel);  /* Force widgets to get windows  */
 XtAppMainLoop(context);   /* Can't get past here !!!       */
}

/****** Callback Routines *******************************************/

void VerifyInput( w, client, call ) /* Reject non-numeric input     */
Widget w; 
XtPointer client; 
XtPointer call;  
{
 int i;
 XmTextVerifyCallbackStruct *cbs = (XmTextVerifyCallbackStruct*) call;
 for ( i=0; i<cbs->text->length; i++){
   if(cbs->text->ptr[i] < '0' || cbs->text->ptr[i] > '9'){
     cbs->doit = False;
     return;
   }
 }
} 

/********************************************************************/

void NumberInput( w, client, call )
Widget w; 
XtPointer client; 
XtPointer call;  
{
 char *string;
 XtVaGetValues(w, XmNvalue, &string, NULL);
 sscanf(string, "%i", &number);
 XtFree(string);
}  

/********************************************************************/

void StepUp( w, client, call )  /* Increment number and display it */
Widget w; 
XtPointer client; 
XtPointer call; 
{
  char output_string[20];
  number++;
  sprintf(output_string, "%i", number);
  XtVaSetValues( textf, XmNvalue, output_string, NULL); 
}

/********************************************************************/

void StepDown( w, client, call )  /* Decrement number and display it */
Widget w; 
XtPointer client; 
XtPointer call; 
{
 char output_string[20];
 if(number != 0) number--; 
 sprintf(output_string, "%i", number); 
 XtVaSetValues( textf, XmNvalue, output_string, NULL); 
} 
